@@ -21,6 +21,8 @@
 /// );
 /// ```
 use chrono::Utc;
+use image::codecs::png::{CompressionType, FilterType};
+use num_cpus;
 use regex::Regex;
 use std::fs::{File, OpenOptions};
 use std::io::Write;
@@ -30,7 +32,6 @@ use std::str;
 
 const BUILD_OUT_FILE: &str = "constants.rs";
 const SOURCE_DIR: &str = "./src";
-
 const TOML: &'static str = include_str!("Cargo.toml");
 
 fn main() {
@@ -42,6 +43,9 @@ fn main() {
         .open(dest_path)
         .expect(&format!("Can't open '{}'", BUILD_OUT_FILE));
 
+    let module_doc_string = "//! Global constants, such as compiler version used, algorithms, compression and filters supported and others";
+    writeln!(file, "{}", module_doc_string).unwrap();
+
     let now_utc = Utc::now();
     write_str_constant_to_file(
         &mut file,
@@ -94,20 +98,69 @@ fn main() {
         .as_str();
     write_str_constant_to_file(&mut file, "COMPILER_VERSION", &compiler_version);
 
-    let re = Regex::new("image = \"(.*)\"").unwrap();
+    let re = Regex::new("image.*version = \"(.*)\",").unwrap();
     let libpng_version = format!(
         "image-{}",
         re.captures(TOML).unwrap().get(1).unwrap().as_str()
     );
     write_str_constant_to_file(&mut file, "LIBPNG_VERSION", &libpng_version);
 
-    write_vec_constant_to_file(&mut file, "FEATURES", vec!["cpu", "python"]);
+    let mut features = vec!["cpu"];
+
+    if cfg!(feature = "wasm-extension") {
+        features.push("wasm")
+    }
+
+    if cfg!(feature = "python-extension") {
+        features.push("python")
+    }
+
+    write_vec_constant_to_file(&mut file, "FEATURES", features);
 
     write_str_constant_to_file(
         &mut file,
         "PLATFORM_CPU_BITS",
         &(std::mem::size_of::<usize>() * 8).to_string(),
     );
+
+    let libpng_filter_types = vec![
+        FilterType::NoFilter,
+        FilterType::Avg,
+        FilterType::Paeth,
+        FilterType::Sub,
+        FilterType::Up,
+    ];
+    write_enum_variants_to_file(&mut file, "FILTER_TYPES", libpng_filter_types);
+
+    let libpng_compression_types = vec![
+        CompressionType::Default,
+        CompressionType::Best,
+        CompressionType::Fast,
+        CompressionType::Huffman,
+        CompressionType::Rle,
+    ];
+    write_enum_variants_to_file(&mut file, "COMPRESSION_TYPES", libpng_compression_types);
+
+    write_constant_to_file(&mut file, "DEFAULT_THREAD_POOL_SIZE", num_cpus::get());
+
+    write_constant_to_file(&mut file, "MAX_THREAD_POOL_SIZE", num_cpus::get() * 10);
+}
+
+fn write_constant_to_file<T>(file: &mut File, key: &str, val: T)
+where
+    T: std::fmt::Display,
+{
+    writeln!(
+        file,
+        "pub const {}: {} = {};",
+        key,
+        std::any::type_name::<T>(),
+        val
+    )
+    .expect(&format!(
+        "Failed to write '{}' to 'build_constants.rs'",
+        key
+    ));
 }
 
 fn write_str_constant_to_file(file: &mut File, key: &str, val: &str) {
@@ -117,7 +170,10 @@ fn write_str_constant_to_file(file: &mut File, key: &str, val: &str) {
     ));
 }
 
-fn write_vec_constant_to_file(file: &mut File, key: &str, vec: Vec<&str>) {
+fn write_vec_constant_to_file<T>(file: &mut File, key: &str, vec: Vec<T>)
+where
+    T: std::fmt::Display,
+{
     let mut list_str = String::new();
     for value in &vec {
         list_str.push_str(&format!("\"{}\",", value))
@@ -125,8 +181,32 @@ fn write_vec_constant_to_file(file: &mut File, key: &str, vec: Vec<&str>) {
     list_str.pop();
     writeln!(
         file,
-        "pub const {}: [&'static str; {}] = [{}];",
+        "pub const {}: [{}; {}] = [{}];",
         key,
+        std::any::type_name::<T>(),
+        vec.len(),
+        list_str
+    )
+    .expect(&format!(
+        "Failed to write '{}' to 'build_constants.rs'",
+        key
+    ));
+}
+
+fn write_enum_variants_to_file<T>(file: &mut File, key: &str, vec: Vec<T>)
+where
+    T: std::fmt::Debug,
+{
+    let mut list_str = String::new();
+    for value in &vec {
+        list_str.push_str(&format!("{}::{:?},", std::any::type_name::<T>(), value))
+    }
+    list_str.pop();
+    writeln!(
+        file,
+        "pub const {}: [{}; {}] = [{}];",
+        key,
+        std::any::type_name::<T>(),
         vec.len(),
         list_str
     )
